package net.sf.sido.spring.mvc;

import net.sf.sido.support.DataFactories;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.parsing.BeanComponentDefinition;
import org.springframework.beans.factory.support.AbstractBeanDefinition;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.RootBeanDefinition;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.w3c.dom.Element;

public class SiDOMVCConfigurationParser implements BeanDefinitionParser {

    public static final String ATTR_REF = "ref";
    public static final String ATTR_POJO = "pojo";

    public static final String DEFAULT_DATA_FACTORY_ID = "sidoDataFactory";
    public static final String DEFAULT_SIDO_SPRING_MVC_ADAPTER = "sidoSpringMVCAdapter";

    @Override
    public BeanDefinition parse(Element element, ParserContext parserContext) {

        // Factory reference ?
        String ref = element.getAttribute(ATTR_REF);
        if (StringUtils.isBlank(ref)) {
            // Creates the factory definition
            RootBeanDefinition factoryBeanDefinition = new RootBeanDefinition(DataFactories.class);
            factoryBeanDefinition.setFactoryMethodName("typedFactory");
            parserContext.registerBeanComponent(
                    new BeanComponentDefinition(
                            factoryBeanDefinition,
                            DEFAULT_DATA_FACTORY_ID));
            ref = DEFAULT_DATA_FACTORY_ID;
        }

        // Messages
        registerMessageConverter(parserContext, JSONDataObjectHttpMessageConverter.class, ref,
                "jsonDataObjectHttpMessageConverter");
        registerMessageConverter(parserContext, XMLDataObjectHttpMessageConverter.class, ref,
                "xmlDataObjectHttpMessageConverter");

        // Argument resolvers
        // DataObject
        parserContext.registerBeanComponent(
                new BeanComponentDefinition(
                        BeanDefinitionBuilder.rootBeanDefinition(DataObjectArgumentResolver.class)
                                .addConstructorArgReference(ref)
                                .getRawBeanDefinition(),
                        "dataObjectArgumentResolver"));
        // DOObject
        parserContext.registerBeanComponent(
                new BeanComponentDefinition(
                        BeanDefinitionBuilder.rootBeanDefinition(DOObjectArgumentResolver.class)
                                .getRawBeanDefinition(),
                        "doObjectArgumentResolver"));

        // Adapter
        BeanDefinitionBuilder adapterBuilder = BeanDefinitionBuilder.rootBeanDefinition(
                SiDOSpringMVCAdapter.class);
        adapterBuilder.setInitMethodName("init");
        adapterBuilder.setAutowireMode(AbstractBeanDefinition.AUTOWIRE_CONSTRUCTOR);
        parserContext.registerBeanComponent(
                new BeanComponentDefinition(
                        adapterBuilder.getRawBeanDefinition(),
                        DEFAULT_SIDO_SPRING_MVC_ADAPTER));

        // OK
        return null;
    }

    private <T> void registerMessageConverter(ParserContext parserContext, Class<T> converterClass,
                                              String dataFactoryRef, String converterName) {
        parserContext.registerBeanComponent(
                new BeanComponentDefinition(
                        BeanDefinitionBuilder.rootBeanDefinition(converterClass)
                                .addConstructorArgReference(dataFactoryRef)
                                .getRawBeanDefinition(),
                        converterName));
    }
}
